/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <string.h>
#include <config.h>
#include <libxfcegui4/libxfcegui4.h>

#include "entry_edit_dialog.h"
#include "entry.h"
#include "tag.h"
#include "debug.h"
#include "treeview.h"
#include "node.h"

#define RED 0
#define GREEN 1
#define WHITE 2

const char *label_types[] = {"Article", "Book", "Booklet", "Conference", "Inbook",
	"Incollection", "Inproceedings", "Manual", "Mastersthesis", "Misc",
	"PhdThesis", "Proceedings", "Techreport", "Unpublished"};

const char *label_entry[] = {"Address", "Annote", "Author", "Booktitle", "Chapter",
	"Crossref", "Edition", "Editor", "E-print", "How Published",
	"Institution", "Journal", "Key", "Month", "Note",
	"Number", "Organization", "Pages", "Publisher", "School",
	"Series", "Title", "Type", "URL", "Volume", "Year"};

const char *tooltips[] = { "Address of publisher",
	"Annotation for annotated bibliography styles", 
	"Name(s) of the author(s), separated by 'and' if more than one", 
	"Title of the book, if only part of it is being cited", 
	"Chapter number", 
	"Citation key of the cross-referenced entry", 
	"Edition of the book (such as \"first\" or \"second\")", 
	"Name(s) of the editor(s), separated by 'and' if more than one", 
	"Specification of electronic publication", 
	"Publishing method if the method is nonstandard", 
	"Institution that was involved in the publishing", 
	"Journal or magazine in which the work was published", 
	"Hidden field used for specifying or overriding the alphabetical order of entries", 
	"Month of publication or creation if unpublished", 
	"Miscellaneous extra information", 
	"Number of journal, magazine, or tech-report", 
	"Sponsor of the conference", 
	"Page numbers separated by commas or double-hyphens", 
	"Name of publisher", 
	"School where thesis was written", 
	"Series of books in which the book was published", 
	"Title of the work", 
	"Type of technical report", 
	"Internet address", 
	"Number of the volume", 
	"Year of publication or creation if unpublished"};
	
	

static void set_color_base(GtkWidget *entry, int selected_color)
{
	_DEBUG(("set_color"));
	
	GdkColor color;
	
	if (selected_color == RED) {
		color.red = 0xe7ff;
		color.green = 0xb3ff;
		color.blue = 0xb3ff;	
	} else if (selected_color == GREEN) {
		color.red = 0xcdff;
		color.green = 0xe7ff;
		color.blue = 0xb3ff;
	} else if (selected_color == WHITE) {
		color.red = 0xffff;
		color.green = 0xffff;
		color.blue = 0xffff;
	}

	gtk_widget_modify_base(entry, GTK_STATE_NORMAL, &color);
}

static int change_color(GtkEntry *entry)
{
	_DEBUG(("change_color"));
	
	if (strcmp(gtk_entry_get_text(entry), "") == 0) {
		set_color_base(GTK_WIDGET(entry), RED);
		return 0;
	} else {
		set_color_base(GTK_WIDGET(entry), GREEN);
		return 1;
	}
}

static int get_entry_pos(char *str)
{
	_DEBUG(("get_entry_pos"));
	int i;
	if(str != NULL) {
		for (i = 0; i < N_ENTRIES; i++) {
			if (strcasecmp(str, label_entry[i]) == 0)
				return i;
		}
	}
	return -1;
}

static void entry_colors(struct edit_entry *entries, gint type)
{
	_DEBUG(("entry_colors"));
	
	set_color_base(entries[get_entry_pos("author")].entry, WHITE);
	set_color_base(entries[get_entry_pos("editor")].entry, WHITE);
	set_color_base(entries[get_entry_pos("chapter")].entry, WHITE);
	set_color_base(entries[get_entry_pos("title")].entry, WHITE);
	set_color_base(entries[get_entry_pos("journal")].entry, WHITE);
	set_color_base(entries[get_entry_pos("year")].entry, WHITE);
	set_color_base(entries[get_entry_pos("publisher")].entry, WHITE);
	set_color_base(entries[get_entry_pos("booktitle")].entry, WHITE);
	set_color_base(entries[get_entry_pos("pages")].entry, WHITE);
	set_color_base(entries[get_entry_pos("school")].entry, WHITE);
	set_color_base(entries[get_entry_pos("institution")].entry, WHITE);
	set_color_base(entries[get_entry_pos("note")].entry, WHITE);
	
	switch(type) {
		case ARTICLE:
			_DEBUG(("Article"));
			change_color(GTK_ENTRY(entries[get_entry_pos("author")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("title")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("journal")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("year")].entry));
			break;
		case BOOK:
			_DEBUG(("book"));
			if (!change_color(GTK_ENTRY(entries[get_entry_pos("author")].entry)))
				if (change_color(GTK_ENTRY(entries[get_entry_pos("editor")].entry)))
					set_color_base(entries[get_entry_pos("author")].entry, WHITE);
			change_color(GTK_ENTRY(entries[get_entry_pos("title")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("publisher")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("year")].entry));
			break;
		case BOOKLET:
			_DEBUG(("booklet"));
			change_color(GTK_ENTRY(entries[get_entry_pos("title")].entry));
			break;
		case CONFERENCE:
		case INCOLLECTION:
		case INPROCEEDINGS:
			_DEBUG(("conference/incollection/inproceedings"));
			change_color(GTK_ENTRY(entries[get_entry_pos("author")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("title")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("booktitle")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("year")].entry));
			break;
		case INBOOK:
			_DEBUG(("inbook"));
			if (!change_color(GTK_ENTRY(entries[get_entry_pos("author")].entry)))
				if (change_color(GTK_ENTRY(entries[get_entry_pos("editor")].entry)))
					set_color_base(entries[get_entry_pos("author")].entry, WHITE);
			change_color(GTK_ENTRY(entries[get_entry_pos("title")].entry));
			if (!change_color(GTK_ENTRY(entries[get_entry_pos("chapter")].entry)))
				if (change_color(GTK_ENTRY(entries[get_entry_pos("pages")].entry)))
					set_color_base(entries[get_entry_pos("chapter")].entry, WHITE);
			change_color(GTK_ENTRY(entries[get_entry_pos("publisher")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("year")].entry));
			break;
		case MANUAL:
			_DEBUG(("manual"));
			change_color(GTK_ENTRY(entries[get_entry_pos("title")].entry));
			break;
		case MASTERSTHESIS:
			_DEBUG(("mastersthesis"));
			change_color(GTK_ENTRY(entries[get_entry_pos("author")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("title")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("school")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("year")].entry));
			break;
		case MISC:
			_DEBUG(("misc"));
			break;
		case PHDTHESIS:
			_DEBUG(("phdthesis"));
			change_color(GTK_ENTRY(entries[get_entry_pos("title")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("year")].entry));
			break;
		case TECHREPORT:
			_DEBUG(("techreport"));
			change_color(GTK_ENTRY(entries[get_entry_pos("author")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("title")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("institution")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("year")].entry));
			break;
		case UNPUBLISHED:
			_DEBUG(("unpublished"));
			change_color(GTK_ENTRY(entries[get_entry_pos("author")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("title")].entry));
			change_color(GTK_ENTRY(entries[get_entry_pos("note")].entry));
	}
}

static void entry_sensitivity(struct edit_entry *entries, gint type)
{
	/* These entries is not used by any of the standard types in bibtex */
	//gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("url")].entry), FALSE);
	//gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("e-print")].entry), FALSE);
	//gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("crossref")].entry), FALSE);
	//gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("annote")].entry), FALSE);
	
	/* These entries is use by all types */
	//gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("title")].entry), TRUE);
	//gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("month")].entry), TRUE);
	//gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("year")].entry), TRUE);
	//gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("key")].entry), TRUE);
	//gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("note")].entry), TRUE);
	
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("author")].entry), type == PROCEEDINGS ? FALSE : TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("editor")].entry), type == BOOK || type == CONFERENCE || type == INBOOK || type == INCOLLECTION || type == INPROCEEDINGS || type == PROCEEDINGS ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("institution")].entry), type == TECHREPORT || type == MASTERSTHESIS ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("organization")].entry), type == CONFERENCE || type == INCOLLECTION || type == INPROCEEDINGS || type == MANUAL || type == PROCEEDINGS ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("school")].entry), type == MASTERSTHESIS || type == PHDTHESIS ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("type")].entry), type == TECHREPORT ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("booktitle")].entry), type == CONFERENCE || type == INCOLLECTION || type == INPROCEEDINGS ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("chapter")].entry), type == INBOOK ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("edition")].entry), type == BOOK || type == INBOOK || type == MANUAL ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("number")].entry), type == ARTICLE || type == TECHREPORT ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("pages")].entry), type == ARTICLE || type == CONFERENCE || type == INBOOK || type == INCOLLECTION || type == INPROCEEDINGS ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("volume")].entry), type == ARTICLE || type == BOOK || type == INBOOK ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("address")].entry), type == ARTICLE || type == MISC || type == UNPUBLISHED ? FALSE : TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("how published")].entry), type == BOOKLET || type == MISC ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("journal")].entry), type == ARTICLE ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("publisher")].entry), type == BOOK || type == CONFERENCE || type == INBOOK || type == INCOLLECTION || type == INPROCEEDINGS || type == PROCEEDINGS ? TRUE : FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(entries[get_entry_pos("series")].entry), type == BOOK || type == INBOOK ? TRUE : FALSE);
}

static void combobox_changed_event(GtkComboBox *widget, struct edit_entry *edit_entry)
{
	_DEBUG(("combobox_changed_event"));
	entry_colors(edit_entry, gtk_combo_box_get_active(widget));
	entry_sensitivity(edit_entry, gtk_combo_box_get_active(widget));
}

static void entry_changed_event(GtkEntry *widget, GtkComboBox *combobox)
{
	_DEBUG(("entry_changed_event"));
	g_signal_emit_by_name(G_OBJECT (combobox), "changed");
}

static GtkWidget *create_hbox(GtkWidget **combobox, GtkWidget **key_entry, gpointer edit_entry)
{
	_DEBUG(("create_hbox"));
	GtkWidget *hbox;
	int i;

	*key_entry = gtk_entry_new();
	gtk_entry_set_width_chars(GTK_ENTRY (*key_entry), 25);
	gtk_widget_set_tooltip_text(*key_entry, _("Key used when citing the publication"));

	*combobox = gtk_combo_box_new_text();
	for (i = 0; i < N_TYPES; i++)
		gtk_combo_box_append_text (GTK_COMBO_BOX (*combobox),label_types[i] );
	
	g_signal_connect (G_OBJECT (GTK_COMBO_BOX(*combobox)), "changed",
                      G_CALLBACK (combobox_changed_event), edit_entry);

	//gtk_combo_box_set_active (GTK_COMBO_BOX (combobox), 0);
	//combobox_changed_event(GTK_COMBO_BOX(combobox), note);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Type: ")), TRUE, FALSE, 10);
	gtk_box_pack_start(GTK_BOX(hbox), *combobox, TRUE, FALSE, 10);
	gtk_box_pack_end(GTK_BOX(hbox), *key_entry, TRUE, FALSE, 10);
	gtk_box_pack_end(GTK_BOX(hbox), gtk_label_new(_("Key: ")), TRUE, FALSE,10);
	
	return hbox;
}

static int get_type_pos(char *str)
{
	_DEBUG(("get_type_pos"));
	int i;
	if(str != NULL) {
		for (i = 0; i < N_TYPES; i++) {
			if (strcasecmp(str, label_types[i]) == 0)
				return i;
		}
	}
	return -1;
}

static void set_entry_text(GtkEntry *entry, char *text)
{
	_DEBUG(("set_entry_text"));
	if (text != NULL && entry != NULL)
		gtk_entry_set_text(entry, text);
}

void show_entry_edit_dialog(struct xfbib *xfbib, struct entry *selected)
{
	_DEBUG(("show_entry_edit_dialog"));
	GtkWidget *dialog, *notebook, *hbox, *key_entry, *combobox, *vbox[4];
	struct edit_entry edit_entry[N_ENTRIES];
	int i, pos;
	struct tag *tags;
	char *type;
	const gchar *value, *key;
	struct node *node = xfbib->list;
	
	xfce_textdomain (PACKAGE, LOCALEDIR, "ISO-8859-1");

	dialog = xfce_titled_dialog_new_with_buttons ((selected == NULL) ? _("Add entry") : _("Edit entry"), NULL,
						GTK_DIALOG_MODAL,
						GTK_STOCK_OK, GTK_RESPONSE_OK,
						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
						NULL);
	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
	
	gtk_window_set_icon_name(GTK_WINDOW(dialog), "xfbib");

	hbox = create_hbox(&combobox, &key_entry, edit_entry);
	
	for (i = 0; i < 4; i++)
		vbox[i] = gtk_vbox_new(FALSE, 10);

	for (i = 0; i < N_ENTRIES; i++)  {
		edit_entry[i].label = gtk_label_new(label_entry[i]);
		edit_entry[i].entry = gtk_entry_new();
		gtk_entry_set_width_chars(GTK_ENTRY (edit_entry[i].entry), 30);
		gtk_widget_set_tooltip_text (edit_entry[i].entry, tooltips[i]);
		edit_entry[i].hbox = gtk_hbox_new(FALSE, 10);
		gtk_box_pack_start(GTK_BOX(edit_entry[i].hbox), edit_entry[i].label, FALSE, FALSE, 10);
		gtk_box_pack_end(GTK_BOX(edit_entry[i].hbox), edit_entry[i].entry, FALSE, TRUE, 10);
		g_signal_connect (G_OBJECT (GTK_ENTRY(edit_entry[i].entry)), "changed",
                      G_CALLBACK (entry_changed_event), combobox);
		
		if (i == AUTHOR || i == EDITOR || i == INSTITUTION || i == ORGANIZATION || i == SCHOOL || i == TITLE || i == TYPE)
			gtk_box_pack_start (GTK_BOX (vbox[0]), edit_entry[i].hbox, FALSE, FALSE, 2);
		else if (i == BOOKTITLE || i == CHAPTER || i == EDITION || i == MONTH || i == NUMBER || i == PAGES || i == VOLUME || i == YEAR)
			gtk_box_pack_start (GTK_BOX (vbox[1]), edit_entry[i].hbox, FALSE, FALSE, 2);
		else if (i == ADDRESS || i == HOWPUBLISHED || i == JOURNAL || i == PUBLISHER || i == SERIES)
			gtk_box_pack_start (GTK_BOX (vbox[2]), edit_entry[i].hbox, FALSE, FALSE, 2);
		else
			gtk_box_pack_start (GTK_BOX (vbox[3]), edit_entry[i].hbox, FALSE, FALSE, 2);
	}


	notebook = gtk_notebook_new ();
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
	
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox[0], gtk_label_new(_("Title")));	
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox[1], gtk_label_new(_("Publication")));	
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox[2], gtk_label_new(_("Publisher")));	
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox[3], gtk_label_new(_("Misc")));	

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 10);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), notebook, TRUE, TRUE, 10);

	gtk_widget_show_all(dialog);

	if (selected != NULL) {
		if ((pos = get_type_pos(selected->type)) != -1) {
			/* Adding predefined values */
			gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), pos);
			set_entry_text(GTK_ENTRY(key_entry), selected->key);
			for (tags = selected->tags; tags != NULL; tags = tags->next) {
				pos = get_entry_pos(tags->name);
				set_entry_text(GTK_ENTRY(edit_entry[pos].entry), tags->value);
			}
		} else
			_DEBUG(("Type equals -1"));
	}
	
	while ( TRUE ) {
		if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
			_DEBUG(("Response OK"));
			key = gtk_entry_get_text(GTK_ENTRY(key_entry));
			if ((type = gtk_combo_box_get_active_text(GTK_COMBO_BOX(combobox))) == NULL) {
				_DEBUG(("No type selected"));
				xfce_warn(_("No type was selected"));
				continue;
			} else if (strcmp(key, "") == 0) {
				_DEBUG(("No key selected"));
				xfce_warn(_("No key was selected"));
				continue;
			} else {
				if (selected == NULL) {
					_DEBUG(("Add new entry"));
					/* TODO: Maybe you shouldnt have to set the type and key */
					selected = add_entry(&node, type,(char*) key);
				} else {
					_DEBUG(("Edit old entry"));
					selected->key = strdup(key);
					selected->type = strdup(type);
					free_tags(selected->tags);
				}
				for (i = 0; i < N_ENTRIES; i++) {
					value = gtk_entry_get_text(GTK_ENTRY(edit_entry[i].entry));
					if(GTK_WIDGET_IS_SENSITIVE(GTK_WIDGET(edit_entry[i].entry)) && (strcmp(value, "") != 0)) {
						fprintf(stderr, "label = %s\tvalue = %s\n",
								gtk_label_get_text(GTK_LABEL(edit_entry[i].label)), value);
						/* Field was not empty and sensitive, so add value to a new entry */
						add_tag(selected,(char*) gtk_label_get_text(GTK_LABEL(edit_entry[i].label)),(char*) value);
					}
				}
				update_treeview(xfbib);
				break;
			}
		} else {
			_DEBUG(("Response Cancel"));
			break;
		}
	}
	gtk_widget_destroy (dialog);
}
